#!/usr/bin/perl -w
use strict;
use lib ".";
use VOPL;
use XML::LibXSLT;
use XML::LibXML;
use File::Path qw(make_path);
use File::Find qw(find);
########################################################
# change these values to your username and password.
my $user='username';
my $passwd='password';
########################################################

my $XSLFILE="veracode_bugzilla.xsl";
my $BUGZILLAPERL='"C:\Program Files\Bugzilla\perl\perl\bin\perl.exe"';
my $IMPORTXML='"C:\Program Files\Bugzilla\bugzilla\importxml.pl"';
#my $BASE_CACHE_DIR="/tmp/cache";
my $BASE_CACHE_DIR="C:\\temp\\cache_runs";
my $SLEEPTIME=60;

# the port and server name to use query the APIs.
my ($server,$port)=('analysiscenter.veracode.com',443);

print "XSLFile=$XSLFILE\n";
print "BUGZILLAPERL=$BUGZILLAPERL\n";
print "IMPORTXML=$IMPORTXML\n";
print "BASE_CACHE_DIR=$BASE_CACHE_DIR\n";
print "SLEEPTIME=$SLEEPTIME\n";

my $vopl=VOPL->new($user,$passwd,$server,$port);
loopcache($vopl,$BASE_CACHE_DIR,@ARGV);

sub loopcache {
	my ($vopl,$base_cache_dir)=(@_);
	do {
		downloadnewresults($vopl,$base_cache_dir,
			readcache($base_cache_dir));
		#print "sleeping\n";
		sleep($SLEEPTIME);
	} while(1);
}
sub readcache {
	my ($basedir)=@_;
	my $res={};
	my @cachefiles;
	find( sub { if($_ eq "cache") { push @cachefiles, $File::Find::name;}},
		$basedir);
	for(@cachefiles) {
		open RFILE, $_ or die "Can't open $_: $!";
		my ($time,$buildid,$appid)=<RFILE>;
		close RFILE;
		chomp $time; chomp $appid; chomp $buildid;
		$res->{"$appid:$buildid"}=$time;
	}
	return $res;
}

sub downloadnewresults {
	my ($vopl,$base_cache_dir,$cacheinfo)=(@_);
	my $res=$vopl->getlistapps(1);
	for my $build (values %$res) {
		# Remove from this array All runs that are not ready.
		$build->{status}= [grep  { $_->{status}=~/Results Ready/; }
					@{$build->{status}}];
	}
	# get a list of all of the builds that have at least 1 ready run.
	my @runs=(grep { scalar(@{$_->{status}})>0 } values %$res);
	for my $run (@runs) {
		my $infotouser=$run->{appname}."(build #".$run->{buildid}.")";
		if(checkrun($run, $cacheinfo)) {
			# print "Updating $infotouser.\n";
			importrun($vopl,$base_cache_dir,$run);
			next;
		}
		#print "no update for $infotouser\n";
	}
}

sub checkrun {
	my ($run,$cacheinfo)=@_;
	my $id=join ":", ($run->{appid},$run->{buildid});
	if(!exists $cacheinfo->{$id}) { return 1; }
	if($cacheinfo->{$id} < $run->{last_published_epoc}) { return 1; }
	return 0;
}
sub importrun {
	my ($vopl,$base,$run)=@_;
	my $appname=$run->{appname};
	$appname=~s/[^\w\d_\.]+/_/g;
	my $buildid=$run->{buildid};
	my $dir="$base/$appname/$buildid";
	print "Updating $appname at $dir\n";
	my $results;
	eval {
		($results)=$vopl->getbuildidsinxml($run->{buildid});
	};
	if($@) { return; }
	if($results eq "") { return; }
	if( ! -d "$dir") {
		make_path("$dir")or die "can't make $dir:$!";
	}

	open WFILE, "> $dir/result.xml" or
		die "can't create $dir/result.xml:$!";
	print WFILE $results;
	close WFILE;
	open CACHEF, "> $dir/cache" or
		die "can't create $dir/cache:$!";
	print CACHEF $run->{last_published_epoc}."\n";
	print CACHEF $run->{buildid}."\n";
	print CACHEF $run->{appid}."\n";
	close CACHEF;
	# Now to do the import
	my $xslt=XML::LibXSLT->new();
	my $source =XML::LibXML->load_xml(location=>"$dir/result.xml");
	my $styledoc =XML::LibXML->load_xml(location=>$XSLFILE, no_cdata=>1);
	my $stylesheet=$xslt->parse_stylesheet($styledoc);

	$results=$stylesheet->transform($source);
	open WFILE, "> $dir/import.xml" or die "can't open $dir/import.xml:$!";
	print WFILE $stylesheet->output_as_bytes($results);
	close WFILE;
	my $import="\"$dir\\import.xml\"";
	$import=~s/\//\\/g;
	print "cmd=cmd /C $BUGZILLAPERL -T $IMPORTXML $import\n";
	`cmd /C $BUGZILLAPERL -T $IMPORTXML $import`;
}
