<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:vc="https://www.veracode.com/schema/reports/export/1.0">
<xsl:output method="xml" indent="no"/>

<!-- Veracode Detailed Report XML XSLT Sample: Bugzilla Export
	(c) Veracode 2010
	 
	This XSLT file exports the Veracode Detailed Report XML format to Bugzilla format.
	
	To use: 
	1. edit the value of the xsl:param urlbase to the URL of your Bugzilla instance
	2. set the value of xsl:param exporter to a valid user in your Bugzilla instance. 
	
	You may also wish to:
	3. add products and components: by default this script maps the Veracode application name
	to a Product and assigns the module (or a fixed value of Dynamic or Manual) to the Component. 
	At a minimum you need to supply default Product and Component values in your Bugzilla instance.
	4. define a keyword: by default this script sets the keyword of all bugs to "security"
	5. review priority and severity assignments based on your own priority and severity definitions.
	
	This sample is provided as-is and no warranty is implied. Veracode customers may use this sample
	and modify it for their own purposes.

-->


  <xsl:param name="urlbase" select="string('http://nosox.veracode.local:8080')"/>
  <xsl:param name="maintainer" select="string('bar@veracode.com')"/>
  <xsl:param name="exporter" select="string('ptibbetts@veracode.com')"/>
  <xsl:param name="delim" select="string(',')"/>
  <xsl:param name="quote" select="string('&quot;')"/>
  <xsl:param name="break" select="string('&#xA;')"/>
  <xsl:variable name="apos">'</xsl:variable>



<xsl:template match="vc:detailedreport">
	<bugzilla>
		<xsl:attribute name="version">3.4.6+</xsl:attribute>
		<xsl:attribute name="urlbase"><xsl:value-of select="$urlbase"/></xsl:attribute>
		<xsl:attribute name="maintainer"><xsl:value-of select="$maintainer"/></xsl:attribute>
		<xsl:attribute name="exporter"><xsl:value-of select="$exporter"/></xsl:attribute>
		<xsl:apply-templates />
	</bugzilla>
</xsl:template>

<xsl:template match="text()"/>

<xsl:template match="vc:detailedreport/vc:severity/vc:category/vc:cwe/vc:staticflaws/vc:flaw">
	<!--you can extend this choose to suppress all but new flaws from being turned into bugs -->
	<xsl:choose>
		<xsl:when test="@remediation_status = 'Fixed'" />
			<!--suppress fixed flaws-->
		<!--xsl:when test="@remediation_status = 'Open'"-->
			<!--suppress open, previously reported flaws-->
		<!--xsl:when test="@remediation_status = 'Reopened'"-->	
			<!--suppress reopened flaws-->
		<xsl:otherwise>
	<bug>
		<alias>
			<xsl:value-of select='concat(normalize-space(../../../../../@build_id),"-",@issueid)' />
		</alias>
		<short_desc>
			<xsl:value-of select='concat("Security: CWE ",@cweid," - ",@categoryname," in ",@module,":",@sourcefile," on line ",@line)'/>
		</short_desc>
		<product>
			<xsl:value-of select="normalize-space(../../../../../@app_name)" />
		</product>
		<version>
			<xsl:value-of select="normalize-space(../../../../../@version)" />
		</version>
		<rep_platform>
			<xsl:value-of select="normalize-space(../../../../../@platform)" />
		</rep_platform>
		<creation_ts>
			<xsl:value-of select="@date_first_occurance" />
		</creation_ts>
		<delta_ts>
			<xsl:value-of select="@date_first_occurance" />		
		</delta_ts>
		<bug_status>
			<xsl:value-of select="@remediation_status" />
		</bug_status>
		<priority>
			<xsl:choose>
				<xsl:when test='@severity="5"'>Highest</xsl:when>
				<xsl:when test='@severity="4"'>High</xsl:when>
				<xsl:when test='@severity="3"'>Normal</xsl:when>
				<xsl:when test='@severity="2"'>Low</xsl:when>
				<xsl:otherwise>Lowest</xsl:otherwise>
			</xsl:choose>
		</priority>
		<bug_severity>		
			<xsl:choose>
				<xsl:when test='@severity="5"'>critical</xsl:when>
				<xsl:when test='@severity="4"'>major</xsl:when>
				<xsl:when test='@severity="3"'>normal</xsl:when>
				<xsl:when test='@severity="2"'>minor</xsl:when>
				<xsl:otherwise>minor</xsl:otherwise>
			</xsl:choose>
		</bug_severity>
		<component>
			<xsl:value-of select='@module' />
		</component>
		<keywords>security</keywords>
		<reporter><xsl:text>Veracode Static Analysis</xsl:text></reporter>
		<long_desc isprivate="0">
			<who>Veracode</who>
			<bug_when>
				<xsl:value-of select='@date_first_occurance' />
			</bug_when>
			<thetext>
				<xsl:value-of select='concat("Source: ",@sourcefile," Line: ",@line," Function: ",@functionprototype,$break,$break,@description)' />
			</thetext>
		</long_desc>
		<xsl:apply-templates select="vc:mitigations/vc:mitigation"/>
	</bug>
	</xsl:otherwise>
	</xsl:choose>
</xsl:template>

 <xsl:template match="vc:severity/vc:category/vc:cwe/vc:dynamicflaws/vc:flaw">
 	<!--you can extend this choose to suppress all but new flaws from being turned into bugs -->
	<xsl:choose>
		<xsl:when test="@remediation_status = 'Fixed'" />
			<!--suppress fixed flaws-->
		<!--xsl:when test="@remediation_status = 'Open'"-->
			<!--suppress open, previously reported flaws-->
		<!--xsl:when test="@remediation_status = 'Reopened'"-->	
			<!--suppress reopened flaws-->
		<xsl:otherwise>
 	<bug>
		<alias>
			<xsl:value-of select='concat(normalize-space(../../../../../@build_id),"-",@issueid)' />
		</alias>
		<short_desc>
			<xsl:value-of select='concat("Security: CWE ",@cweid," - ",@categoryname," at ",@url)'/>
		</short_desc>
		<product>
			<xsl:value-of select="normalize-space(../../../../../@app_name)" />
		</product>
		<version>
			<xsl:value-of select="normalize-space(../../../../../@version)" />
		</version>
		<rep_platform>
			<xsl:value-of select="normalize-space(../../../../../@platform)" />
		</rep_platform>
		<creation_ts>
			<xsl:value-of select="@date_first_occurance" />
		</creation_ts>
		<bug_status>
			<xsl:value-of select="@remediation_status" />
		</bug_status>
		<priority>
			<xsl:choose>
				<xsl:when test='@severity="5"'>Highest</xsl:when>
				<xsl:when test='@severity="4"'>High</xsl:when>
				<xsl:when test='@severity="3"'>Normal</xsl:when>
				<xsl:when test='@severity="2"'>Low</xsl:when>
				<xsl:otherwise>Lowest</xsl:otherwise>
			</xsl:choose>
		</priority>
		<bug_severity>
			<xsl:choose>
				<xsl:when test='@severity="5"'>critical</xsl:when>
				<xsl:when test='@severity="4"'>major</xsl:when>
				<xsl:when test='@severity="3"'>normal</xsl:when>
				<xsl:when test='@severity="2"'>minor</xsl:when>
				<xsl:otherwise>minor</xsl:otherwise>
			</xsl:choose>
		</bug_severity>
		<component>
			<xsl:value-of select='@url' />
		</component>
		<keywords>security</keywords>
		<reporter><xsl:text>Veracode Dynamic Analysis</xsl:text></reporter>
		<long_desc isprivate="0">
			<who>Veracode</who>
			<bug_when>
				<xsl:value-of select='@date_first_occurance' />
			</bug_when>
			<thetext>
				<xsl:value-of select='concat("URL: ",@url,$break,$break,@description)' />
			</thetext>
		</long_desc>
		<xsl:apply-templates select="vc:mitigations/vc:mitigation"/>
	</bug>
	</xsl:otherwise>
	</xsl:choose>
</xsl:template>


 <xsl:template match="vc:severity/vc:category/vc:cwe/vc:manualflaws/vc:flaw">
 	<!--as fixed flaws are not reported for manual flaws, do not need to test for flaw status-->
  	<bug>
		<alias>
			<xsl:value-of select='concat(normalize-space(../../../../../@build_id),"-",@issueid)' />
		</alias>
		<short_desc>
			<xsl:value-of select='concat("Security: CWE ",@cweid," - ",@cwename," at ",@location)'/>
		</short_desc>
		<product>
			<xsl:value-of select="normalize-space(../../../../../@app_name)" />
		</product>
		<version>
			<xsl:value-of select="normalize-space(../../../../../@version)" />
		</version>
		<rep_platform>
			<xsl:value-of select="normalize-space(../../../../../@platform)" />
		</rep_platform>
		<bug_status>New</bug_status>
		<priority>
			<xsl:choose>
				<xsl:when test='@severity="5"'>Highest</xsl:when>
				<xsl:when test='@severity="4"'>High</xsl:when>
				<xsl:when test='@severity="3"'>Normal</xsl:when>
				<xsl:when test='@severity="2"'>Low</xsl:when>
				<xsl:otherwise>Lowest</xsl:otherwise>
			</xsl:choose>
		</priority>
		<bug_severity>				
			<xsl:choose>
				<xsl:when test='@severity="5"'>critical</xsl:when>
				<xsl:when test='@severity="4"'>major</xsl:when>
				<xsl:when test='@severity="3"'>normal</xsl:when>
				<xsl:when test='@severity="2"'>minor</xsl:when>
				<xsl:otherwise>minor</xsl:otherwise>
			</xsl:choose>
		</bug_severity>
		<component>
			<xsl:value-of select='@location' />
		</component>
		<keywords>security</keywords>
		<reporter><xsl:text>Veracode Manual Analysis</xsl:text></reporter>
		<long_desc isprivate="0">
			<who>Veracode</who>
			<bug_when>
				<xsl:value-of select='@date_first_occurance' />
			</bug_when>
			<thetext>
				<xsl:value-of select='concat("Location: ",@location,$break,$break,@description,$break,$break,"Resolution:",@resolution)' />
			</thetext>
		</long_desc>
		<xsl:apply-templates select="vc:mitigations/vc:mitigation"/>
	</bug>
</xsl:template>

<xsl:template match="vc:detailedreport/vc:severity/vc:category/vc:cwe/vc:staticflaws/vc:flaw/vc:mitigations/vc:mitigation">
	<long_desc isprivate="0">
		<who>
			<xsl:value-of select="@user" />
		</who>
		<bug_when>
			<xsl:value-of select='@date' />
		</bug_when>
		<thetext>
			<xsl:value-of select='concat("Mitigation type: ",@action,$break,$break,@description)' />
		</thetext>
	</long_desc>
</xsl:template>

<xsl:template match="vc:detailedreport/vc:severity/vc:category/vc:cwe/vc:dynamicflaws/vc:flaw/vc:mitigations/vc:mitigation">
	<long_desc isprivate="0">
		<who>
			<xsl:value-of select="@user" />
		</who>
		<bug_when>
			<xsl:value-of select='@date' />
		</bug_when>
		<thetext>
			<xsl:value-of select='concat("Mitigation type: ",@action,$break,$break,@description)' />
		</thetext>
	</long_desc>
</xsl:template>


</xsl:stylesheet>